/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.22                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for RENESAS SH7203 Starterkit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define NUM_LEDS        (4)

/****** SFRs and identifiers used for LED demo **********************/

#define PEDRL    (*(volatile unsigned short*) (0xFFFE3A02))
#define PECRL3   (*(volatile unsigned short*) (0xFFFE3A12))
#define PECRL4   (*(volatile unsigned short*) (0xFFFE3A10))
#define PEIORL   (*(volatile unsigned short*) (0xFFFE3A06))

#define PCDRL    (*(volatile unsigned short*) (0xFFFE3902))
#define PCCRL4   (*(volatile unsigned short*) (0xFFFE3910))
#define PCIORL   (*(volatile unsigned short*) (0xFFFE3906))

/****** Assign LEDs to Ports ****************************************/

#define LED0_PORT        (PEDRL)
#define LED1_PORT        (PEDRL)
#define LED2_PORT        (PCDRL)
#define LED3_PORT        (PEDRL)

#define LED0_BIT         (10)
#define LED1_BIT         (12)
#define LED2_BIT         (14)
#define LED3_BIT         (11)

#define LED0_PCRL_REG    (PECRL3)
#define LED1_PCRL_REG    (PECRL4)
#define LED2_PCRL_REG    (PCCRL4)
#define LED3_PCRL_REG    (PECRL3)

#define LED0_PIORL_REG   (PEIORL)
#define LED1_PIORL_REG   (PEIORL)
#define LED2_PIORL_REG   (PCIORL)
#define LED3_PIORL_REG   (PEIORL)

#define LED0_PCRL_MD_POS (PECRL3)
#define LED1_PCRL_MD_POS (PECRL4)
#define LED2_PCRL_MD_POS (PCCRL4)
#define LED3_PCRL_MD_POS (PECRL3)


#define LED0_MASK        (1uL << LED0_BIT)
#define LED1_MASK        (1uL << LED1_BIT)
#define LED2_MASK        (1uL << LED2_BIT)
#define LED3_MASK        (1uL << LED3_BIT)

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

static unsigned char _LEDIsOn(int Index) {
  unsigned char result;
  result = 0;
  switch (Index) {
    case 0:
      result = (LED0_PORT & LED0_MASK) ? 0 : 1;    // LED0 is switched on with low level
      break;
    case 1: 
      result = (LED1_PORT & LED1_MASK) ? 0 : 1;    // LED1 is switched on with low level
      break;
    case 2: 
      result = (LED2_PORT & LED2_MASK) ? 0 : 1;    // LED2 is switched on with low level
      break;
    case 3: 
      result = (LED3_PORT & LED3_MASK) ? 0 : 1;    // LED3 is switched on with low level
      break;
    default:
    break;
  }
  return result;
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  unsigned short Regval;
  //
  // Configure port pins connected to LEDs, set Port mode
  //
  Regval         = ~(0x0FuL << LED0_PCRL_MD_POS);
  LED0_PCRL_REG &= Regval;
  Regval         = ~(0x0FuL << LED1_PCRL_MD_POS);
  LED1_PCRL_REG &= Regval;
  Regval         = ~(0x0FuL << LED2_PCRL_MD_POS);
  LED2_PCRL_REG &= Regval;
  Regval         = ~(0x0FuL << LED3_PCRL_MD_POS);
  LED3_PCRL_REG &= Regval;
  //
  // Set mode of port to output mode
  //
  LED0_PIORL_REG |= LED0_MASK;
  LED1_PIORL_REG |= LED1_MASK;
  LED2_PIORL_REG |= LED2_MASK;
  LED3_PIORL_REG |= LED3_MASK;
  //
  // Initially switch LEDs on for test
  //
  LED0_PORT &= ~(LED0_MASK);    // LED0 is switched on with low level
  LED1_PORT &= ~(LED1_MASK);    // LED1 is switched on with low level
  LED2_PORT &= ~(LED2_MASK);    // LED2 is switched on with low level
  LED3_PORT &= ~(LED3_MASK);    // LED3 is switched on with low level
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  switch (Index) {
    case 0:
      LED0_PORT &= ~(LED0_MASK);    // LED0 is switched on with low level
      break;
    case 1: 
      LED1_PORT &= ~(LED1_MASK);    // LED1 is switched on with low level
      break;
    case 2: 
      LED2_PORT &= ~(LED2_MASK);    // LED2 is switched on with low level
      break;
    case 3: 
      LED3_PORT &= ~(LED3_MASK);    // LED3 is switched on with low level
      break;
    default:
      break;
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  switch (Index) {
    case 0:
      LED0_PORT |= (LED0_MASK);    // LED0 is switched off with high level
      break;
    case 1: 
      LED1_PORT |= (LED1_MASK);    // LED1 is switched off with high level
      break;
    case 2: 
      LED2_PORT |= (LED2_MASK);    // LED2 is switched off with high level
      break;
    case 3: 
      LED3_PORT |= (LED3_MASK);    // LED3 is switched off with high level
      break;
    default:
      break;
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  switch (Index) {
    case 0:
      if(_LEDIsOn(Index)) {
        LED0_PORT |=  (LED0_MASK);    // LED0 is switched off with high level
      } else {
        LED0_PORT &= ~(LED0_MASK);    // LED0 is switched on with low level
    }
      break;
    case 1: 
      if(_LEDIsOn(Index)) {
        LED1_PORT |=  (LED1_MASK);    // LED1 is switched off with high level
      } else {
        LED1_PORT &= ~(LED1_MASK);    // LED1 is switched on with low level
    }
      break;
    case 2: 
      if(_LEDIsOn(Index)) {
        LED2_PORT |=  (LED2_MASK);    // LED2 is switched off with high level
      } else {
        LED2_PORT &= ~(LED2_MASK);    // LED2 is switched on with low level
    }
      break;
    case 3: 
      if(_LEDIsOn(Index)) {
        LED3_PORT |=  (LED3_MASK);    // LED3 is switched off with high level
      } else {
        LED3_PORT &= ~(LED3_MASK);    // LED3 is switched on with low level
    }
      break;
    default:
      break;
  }
}

/****** End Of File *************************************************/
