/* Adapted for use with IAR Embedded Workbench */
/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name     : bus_init.c
* Version       : 1.00
* Device(s)     : SH7269
* Tool-Chain    : HEW, SuperH RISC engine Standard Toolchain v9.4.0.0
* H/W Platform  : RSK2+SH7269
* Description   : External memory bus initialization
*******************************************************************************
* History       : DD.MM.YYYY Version Description
*      : 08.03.2010 Adapted to IAR Embedded Workbench
*               : 03.03.2012 1.00    First Release
******************************************************************************/

/******************************************************************************
System Includes
******************************************************************************/

/******************************************************************************
User Includes (Project level includes)
******************************************************************************/

/* Header file provides a structure to access on-chip I/O registers. */
#include "io7268.h"
/* Header file provides prototype(s) for the function(s) defined in
    this file.    */
#include "bus_init.h"

/******************************************************************************
Compiler Directives
******************************************************************************/
/* '#pragma section' instructs the linker to place all subsequent code in
    the named section. This can be used to relocate code in the memory map.
    Use of the directive without the section name will reset the section
    to default. */

/******************************************************************************
User Program Code
******************************************************************************/

/******************************************************************************
* Function Name : configure_bus_and_memory
* Description   : Configures the external bus and memory.
* Argument      : none
* Return value  : none
******************************************************************************/

void configure_bus_and_memory (void)
{
    
    /* Configure the CS0 space.    */
    init_bsc_cs0();
}

/******************************************************************************
End of function configure_bus_and_memory
******************************************************************************/

/******************************************************************************
* Function Name : init_bsc_cs0
* Description   : configures the NOR flash memory connected to CS0 space.
* Argument      : none
* Return value  : none
******************************************************************************/

void init_bsc_cs0 (void)
{
    
    /* Configure the port pins as external address A24-A21 pins */
    PORT.PBCR5.BIT.PB21MD  = 1u;    /* Set A21  */
    
    /* Configure the port pin PC2 as WRn */
    PORT.PCCR0.BIT.PC2MD   = 1u;    /* Set WRn# */
    
    /* Configure the port pin PC3 as WE0n */
    PORT.PCCR0.BIT.PC3MD   = 1u;    /* Set WE0# */

    /* Configure the CS0 Space Wait Control Register
       for Normal Space, SRAM with Byte Selection, and MPX-I/O

       b1:b0      - HW           - 01 (1.5 Delay cycles RD, WEn Negation to Address,
                                      CS0 Negation)
       b5:b2      - Reserved     - Set to 0000
       b6         - WM           - 1 (External wait input is ignored)
       b10:b7     - WR           - 0110 (6 Wait cycles for Access)
       b12:b11    - SW           - 00 (0.5 cycles of Delay Cycles from Address,
                                      CS0 Assertion to RD, WEn Assertion)
       b19:b13    - Reserved     - Set to 0000000
       b20        - BAS          - 0 (Asserts the WEn signal at the read/write
                                     timing and asserts the RD/WR signal during
                                     the write access cycle)
       b31:b21    - Reserved     - Set to 00000000000 */

    BSC.CS0WCR.NORMAL.LONG = CS0WCR_SETTING;

    /* Configure CS0 Space Bus Control Register (CS0BCR)
       b8:b0      - Reserved     - Set to 000000000
       b10:b9     - BSZ          - 10    (16 bit Data Bus)
       b11        - ENDIAN       - 0     (Big Endian)
       b14:b12    - TYPE         - 001    (Burst ROM (clock asynchronous))
       b15        - Reserved     - Set to 0
       b18:b16    - IWRRS        - 000   (No Idle cycle)
       b21:b19    - IWRRD        - 000   (No Idle cycle)
       b24:b22    - IWRWS        - 000   (No Idle cycle)
       b27:b25    - IWRWD        - 000   (No Idle cycle)
       b30:b28    - IWW          - 001   (1 Idle cycle between Write-Read Cycles and
                                         Write-Write Cycles)
       b31        - Reserved        - Set to 0 */
    BSC.CS0BCR.LONG = CS0BCR_SETTING;
}

/******************************************************************************
End of function init_bsc_cs0
******************************************************************************/

/******************************************************************************
End of File
******************************************************************************/

