/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.22                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Renesas SH7267 CPU board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "iodefine.h"
#include "BSP.h"

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // Switch port function in U21
  //
  PORT.PBCR5.BIT.PB22MD   = 0u;  // Set PB22/bus switch pin mode to I/O usage
  PORT.PBDR1.BIT.PB22DR   = 1u;  // Configure to output
  PORT.PBIOR1.BIT.PB22IOR = 1u;  // Set pin to activate bus switch A ports
  //
  // Setup LED port pins
  //
  PORT.PAIOR0.BIT.PA0IOR  = 1u;  // Set LED0 pins to output
  PORT.PAIOR0.BIT.PA1IOR  = 1u;  // Set LED1 pins to output
  PORT.PADR0.BIT.PA0DR    = 1u;  // LED0 is switched off with high level
  PORT.PADR0.BIT.PA1DR    = 1u;  // LED1 is switched off with high level
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    PORT.PADR0.BIT.PA0DR = 0u;      // LED0 is switched on with low level
  } else if (Index == 1){
    PORT.PADR0.BIT.PA1DR = 0u;
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    PORT.PADR0.BIT.PA0DR = 1;       // LED0 is switched off with high level
  } else if (Index == 1){
    PORT.PADR0.BIT.PA1DR = 1;       // LED1 is switched off with high level
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if (PORT.PADR0.BIT.PA0DR & 0x01u) {  // LED0 is switched on with low level
      BSP_SetLED(Index);                 // Clear LED
    } else {
      BSP_ClrLED(Index);                 // Set LED
    }
  } else if (Index == 1){
    if (PORT.PADR0.BIT.PA1DR & 0x01u) {  // LED1 is switched on with low level
      BSP_SetLED(Index);                 // Clear LED
    } else {
      BSP_ClrLED(Index);                 // Set LED
    }
  }
}

/****** End Of File *************************************************/
