/* Adapted for use with IAR Embedded Workbench */
/*******************************************************************************
*   DISCLAIMER
*
*   This software is supplied by Renesas Electronics Corporation and is only
*   intended for use with Renesas products. No other uses are authorized.
*
*   This software is owned by Renesas Electronics Corporation and is protected under
*   all applicable laws, including copyright laws.
*
*   THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
*   REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
*   INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
*   PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY
*   DISCLAIMED.
*
*   TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
*   ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
*   FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
*   FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
*   AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*
*   Renesas reserves the right, without notice, to make changes to this
*   software and to discontinue the availability of this software.
*   By using this software, you agree to the additional terms and
*   conditions found by accessing the following link:
*   http://www.renesas.com/disclaimer
********************************************************************************
* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.
*""FILE COMMENT""*********** Technical reference data **************************
*   System Name : SH7266/SH7267 Sample Program
*   File Name   : hwsetup.c
*   Abstract    : Hardware initialization function
*   Version     : 1.00.00
*   Device      : SH7266/SH7267
*   Tool-Chain  : High-performance Embedded Workshop (Ver.4.07.00).
*               : C/C++ compiler package for the SuperH RISC engine family
*               :                              (Ver.9.03Release02).
*   OS          : None
*   H/W Platform: R0K57267(CPU board)
*   Description :
********************************************************************************
*   History     : Nov.19,2010 Ver.1.00.00 First Release
*""FILE COMMENT END""**********************************************************/
#include "intrinsics.h"
#include "iodefine.h"

/* ==== Macro name definition ==== */
/* The address when writing in a SDRAM mode register */
#define	SDRAM_MODE		(*(volatile unsigned short *)(0xfffc5040))

/*  CS0 PAGEMODE setting */
//#define PAGEMODE

/* ==== Prototype declaration ==== */
void  HardwareSetup(void);


/*""FUNC COMMENT""**************************************************************
 * ID          :
 * Outline     : CPG settings
 *------------------------------------------------------------------------------
 * Include     : #include "iodefine.h"
 *------------------------------------------------------------------------------
 * Declaration : void io_set_cpg(void);
 *------------------------------------------------------------------------------
 * Description : Clock pulse generator (CPG) is set to set to the internal clock
 *             : (I Clock), peripheral clock (P Clock), bus clock (B Clock), and
 *             : I Clock = 144MHz, B Clock = 72MHz,  P Clock = 36MHz.
 *             : This setting example is the case that the function's input clock
 *             : is 12MHz and clock mode is 0.
 *------------------------------------------------------------------------------
 * Argument    : void
 *------------------------------------------------------------------------------
 * Return Value: void
 *------------------------------------------------------------------------------
 * Note        : None
 *""FUNC COMMENT END""**********************************************************/
static void io_set_cpg(void)
{
	/* ==== CPG Setting ==== */
	CPG.FRQCR.WORD = 0x1103u;		/* PLL1(x12),I:B:P= 12:6:3
									* CKIO:Output at time usually,Output when bus right is opened,output at standby"L"
									* Clockin = 12MHz, CKIO = 72MHz
									* I Clock = 144MHz, B Clock = 72MHz,
									* P Clock = 36MHz
									*/
    /* ---- The clock of all modules is permitted. ---- */
	CPG.STBCR3.BYTE = 0x02u;		/* Port level is keep in standby mode						*/
									/* IEBus, MTU2,SDHI0, SDHI1, A/D, [1], RTClock				*/
	CPG.STBCR4.BYTE = 0x00u;		/* SCIF0, SCIF1, SCIF2, SCIF3, SCIF4, SCIF5, SCIF6, SCIF7	*/
	CPG.STBCR5.BYTE = 0x10u;		/* I2C30, I2C31, I2C32, [1], RCAN0, RCAN1, RSPI0, RSPI1		*/
	CPG.STBCR6.BYTE = 0x00u;		/* SSI0, SSI1, SSI2, SSI3, CD-ROMDEC, SRC0, SRC1, USB		*/
	CPG.STBCR7.BYTE = 0x2au;		/* SIOF, RSPDIF, [1], VDC3, [1], CMT, [1], NAND				*/
	CPG.STBCR8.BYTE = 0x7eu;		/* PWM, [1], [1], [1], [1], [1], [1], DECOMP				*/
	/* ----  Writing to large-capacity RAM is enabled. ---- */
	CPG.SYSCR5.BYTE = 0x0fu;
}

/*""FUNC COMMENT""**************************************************************
 * ID          :
 * Outline     : SDRAM 16 bit bus width connection settings
 *------------------------------------------------------------------------------
 * Include     : #include "iodefine.h"
 *------------------------------------------------------------------------------
 * Declaration : void io_init_sdram(void);
 *------------------------------------------------------------------------------
 * Description : A connection setup to SDRAM of CS3 space.
 *             : The PFC setting is set by bit manipulation not to change the PFC
 *             : set value which is set by other process.
 *------------------------------------------------------------------------------
 * Argument    : void
 *------------------------------------------------------------------------------
 * Return Value: void
 *------------------------------------------------------------------------------
 * Note        : None
 *""FUNC COMMENT END""**********************************************************/
static void io_init_sdram(void)
{
	volatile int j = 133;					/* 200usec wait count	*/

	/* ==== PFC settings ==== */
	PORT.PCCR2.BIT.PC8MD = 1u;				/* CS3#		*/
	PORT.PCCR1.BIT.PC7MD = 1u;				/* CKE		*/
	PORT.PCCR1.BIT.PC6MD = 1u;				/* CAS#		*/
	PORT.PCCR1.BIT.PC5MD = 1u;				/* RAS#		*/
	PORT.PCCR1.BIT.PC4MD = 1u;				/* DQMU#	*/
	PORT.PCCR0.BIT.PC3MD = 1u;				/* DQML#	*/
	PORT.PCCR0.BIT.PC2MD = 1u;				/* RD/WR#	*/

	/* ==== 200us interval elapsed ? ==== */
	while(j-- > 0){
		/* wait */
	}

	/* ==== CS3BCR settings ==== */
	BSC.CS3BCR.LONG = 0x00004400ul;
										/*
											Idle Cycles between Write-read Cycles
											and Write-write Cycles : 0 idle cycles
											Memory type :SDRAM
											Data Bus Size : 16-bit
										*/

	/* ==== CS3WCR settings ==== */
	BSC.CS3WCR.SDRAM.LONG = 0x00002892ul;
										/*
											Precharge completion wait cycles: 1 cycle
											Wait cycles between ACTV command
											and READ(A)/WRITE(A) command : 2 cycles
											CAS latency for Area 3 : 2 cycles
											Auto-precharge startup wait cycles : 1 cycle
											Idle cycles from REF command/self-refresh
											Release to ACTV/REF/MRS command
											: 5 cycles
										*/


	/* ==== SDCR settings ==== */
	BSC.SDCR.LONG = 0x00000809ul;
										/*
											Refresh Control :Refresh
											RMODE :Auto-refresh is performed
											BACTV :Auto-precharge mode
											Row address for Area 3 : 12-bit
											Column Address for Area 3 : 9-bit
										*/

	/* ==== RTCOR settings ==== */
	BSC.RTCOR.LONG = 0xa55a0046ul;		/*
											15.625us/222ns
											= 70(0x46)cycles per refresh
										*/




	/* ==== RTCSR settings ==== */
	BSC.RTCSR.LONG = 0xa55a0010ul;
										/*
											Initialization sequence start
											Clock select B-phy/16
											Refresh count :Once
										*/

	/* ==== Written in SDRAM Mode Register ==== */
	SDRAM_MODE = 0;						/*
											The writing data is arbitrary
											SDRAM mode register setting CS3 space
											Burst read (burst length 1)./Burst write
										*/
}

/*""FUNC COMMENT""**************************************************************
 * ID          :
 * Outline     : CS0 setting
 *------------------------------------------------------------------------------
 * Include     : #include "iodefine.h"
 *------------------------------------------------------------------------------
 * Declaration : void io_init_bsc_cs0(void);
 *------------------------------------------------------------------------------
 * Description : Pin function controller (PFC) and bus state controller (BSC)
 *             : are set, and the access timing to the Flash Memory of CS0 space
 *             : is set.
 *             : The PFC setting is set by bit manipulation not to change the PFC
 *             : set value which is set by other process.
 *------------------------------------------------------------------------------
 * Argument    : void
 *------------------------------------------------------------------------------
 * Return Value: void
 *------------------------------------------------------------------------------
 * Note        : None
 *""FUNC COMMENT END""**********************************************************/
static void io_init_bsc_cs0(void)
{
	/* ==== PFC settings ==== */
	PORT.PBCR5.BIT.PB21MD  = 1u;	/* Set A21 	*/
	PORT.PCCR0.BIT.PC3MD   = 1u;	/* Set WE0# */

#ifdef PAGEMODE

	/* ==== CS0WCR settings ==== */
	BSC.CS0WCR.BROM_ASY.LONG = 0x002203c0ul;
									/* Number of Burst: 4-4 or 2-4-2 			*/
									/* Number of Burst Wait Cycles: 3 cycles 	*/
									/* Number of Access Wait Cycles: 8 cycles 	*/

	/* ==== CS0BCR settings ==== */
	BSC.CS0BCR.LONG = 0x20001400ul;
									/* Idle Cycles between Write-read Cycles 	*/
									/*  and Write-write Cycles: 2 idle cycles 	*/
									/* Type:Burst ROM (ASY) 					*/
									/* Data Bus Size: 16-bit					*/
#else /* PAGEMODE */

	/* ==== CS0WCR settings ====  */
	BSC.CS0WCR.NORMAL.LONG = 0x00000b40ul;
									/* Number of Delay Cycles from Address, 	*/
									/*  CS0# Assertion to RD#,WEn Assertion 	*/
									/*  : 1.5 cycles 							*/
									/* Number of Access Wait Cycles: 6 cycles 	*/
									/* Delay Cycles from RD,WEn# negation to 	*/
									/*  Address,CSn# negation: 0.5 cycles 		*/

	/* ==== CS0BCR settings ==== */
	BSC.CS0BCR.LONG = 0x12400400ul;
									/* Idle Cycles between Write-read Cycles 	*/
									/*  and Write-write Cycles : 1 idle cycle 	*/
									/*  and read-write Cycles : 1 idle cycle 	*/
									/* Data Bus Size: 16-bit 					*/
#endif /* PAGEMODE */
}

/*""FUNC COMMENT""**************************************************************
 * ID          :
 * Outline     : Cache initialization
 *------------------------------------------------------------------------------
 * Include     : #include "iodefine.h"
 *------------------------------------------------------------------------------
 * Declaration : void io_init_cache(void);
 *------------------------------------------------------------------------------
 * Description : Instruction/operand cache are flushed and enabled.
 *             : The section name of this function is changed to be placed in
 *             : the cache-disabled.
 *             : When this function is used only in the state of interrupt level 15,
 *             : the setting and clearing of interrupt mask need not be processed.
 *------------------------------------------------------------------------------
 * Argument    : void
 *------------------------------------------------------------------------------
 * Return Value: void
 *------------------------------------------------------------------------------
 * Note        : None
 *""FUNC COMMENT END""**********************************************************/
static void io_init_cache(void)
{
	volatile unsigned long reg;
	int mask;

	/* ==== Interrupt mask setting ==== */
	mask = _builtin_get_imask();
	_builtin_set_imask(15);					/* Set to the level 15 */

	/* ==== Cache register setting ==== */
	CCNT.CCR1.LONG = 0x0909ul;	/* Write back ON */

									/*
										ICF=1:Instruction cache flushed
										ICE=1:Instruction cache enabled
										OCF=1:Operand cache flushed
										OCE=1:Operand cache enabled
									*/
	/* ==== Reading cache register ==== */
	reg = CCNT.CCR1.LONG ;

	/* ==== Clearing interrupt mask ==== */
	_builtin_set_imask(mask);					/* Set to the original level */

}

/*""FUNC COMMENT""**************************************************************
 * ID          :
 * Outline     : Write-back of cache
 *------------------------------------------------------------------------------
 * Include     : #include "iodefine.h"
 *------------------------------------------------------------------------------
 * Declaration : int io_cache_writeback(void);
 *------------------------------------------------------------------------------
 * Description : All lines of operand cache are disabled, and the contents of
 *             : cache memory are written back to the external memory.
 *             : It has nothing to do with the write-through mode.
 *------------------------------------------------------------------------------
 * Argument    : void
 *------------------------------------------------------------------------------
 * Return Value: 0 : Normal completion
 *------------------------------------------------------------------------------
 * Note        : None
 *""FUNC COMMENT END""**********************************************************/
static int io_cache_writeback(void)
{
	volatile unsigned long *arry;
	unsigned int i,j;
	int mask;

	/* ==== Interrupt mask setting ==== */
	mask = _builtin_get_imask();
	_builtin_set_imask(15);					/* Set to the level 15 */

	/* ====  All entries disabled ==== */
	for(i=0u; i <4u; i++){
		for(j=0u; j < 128u; j++){
			/* ---- Creating an address array address ---- */
			arry = (volatile unsigned long *)(0xf0800000 | (i<<11) | (j<<4));
			/* ---- Write U=0 and V=0 in the address array ---- */
			*arry &= 0xfffffffcul;		/* V=0,U=0 */
		}
	}

	/* ==== Interrupt mask recovery ==== */
	_builtin_set_imask(mask);				/* Set to the original level */

	return 0;
}

//#pragma section=ResetPRG
/*""FUNC COMMENT""**************************************************************
 * ID          :
 * Outline     : Hardware initialization function
 *------------------------------------------------------------------------------
 * Include     : #include "iodefine.h"
 *------------------------------------------------------------------------------
 * Declaration : void  HardwareSetup(void);
 *------------------------------------------------------------------------------
 * Description : The initial settings of CPG, PFC, and BSC (Flash memory
 *             : access control and SDRAM initialization) are processed.
 *------------------------------------------------------------------------------
 * Argument    : void
 *------------------------------------------------------------------------------
 * Return Value: void
 *------------------------------------------------------------------------------
 * Note        : None
 *""FUNC COMMENT END""**********************************************************/
void  HardwareSetup(void)
{
	/*====CPG setting====*/
	io_set_cpg();

	/*====CS0 initialization====*/
	io_init_bsc_cs0();

	/*====SDRAM area initialization====*/
	io_init_sdram();

	/*====Cache setting====*/
	io_init_cache();

  io_cache_writeback();
}


/* End of File */
